/*
    \brief Deklarationen der Funktionen für das Logging der Bibliothek.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_LOGGING_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_LOGGING_H
#endif

#include "df_defines.h"

/*!
    \brief Schaltet die Logfunktionalitaet der Bibliothek ein.

    Loggen von Kommunikationsabläufen in die angegebene Datei.

    \ingroup log

    \export{_DFCLogOn@4, DFCLogOn}

    \deprecated Wird bereits seit Version 02.00.03 nicht mehr unterstützt und ist nur aus Gründen der
    Abwärtskompatibilität enthalten.

    \param[in] filePath Zeichenfolge des Pfades und/oder Dateinamens.

    \retval 0 Datei konnte nicht erstellt oder geöffnet werden.
    \retval 1 Logfunktionalität wurde erfolgreich eingeschaltet.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCLogOn( const char *filePath );

/*!
    \brief Schaltet die Logfunktionalität der Verbindungsnummer ein.

    Loggen von Kommunikationsabläufen der Verbindungsnummer einschalten.

    \ingroup log

    \export{_DFCSetLogOn@4, DFCSetLogOn}

    \note Die Kommunikationsbibliothek loggt alle Kommunikationsabläufe automatisch in entsprechende Dateien. Daher ist
    nicht notwendig über diese Funktion ein Extralog zu erstellen. Bitte sehen Sie für nähere Informationen
    in der Dokumentation zum Thema Hintergrundloggen nach.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCSetLogOn( int connectionNumber );


/*!
    \brief Schaltet die Logfunktionalität der Bibliothek aus.

    Abschalten der Logfunktionalitäg.

    \ingroup log

    \export{_DFCLogOff@0, DFCLogOff}

    \deprecated Wird bereits seit Version 02.00.03 nicht mehr unterstützt und ist nur aus Gründen der
    Abwärtskompatibilität enthalten.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCLogOff();

/*!
    \brief Schaltet die Logfunktionalität der Verbindungsnummer aus.

    Loggen von Kommunikationsabläufen der Verbindungsnummer abschalten.

    \ingroup log

    \export{_DFCSetLogOff@4, DFCSetLogOff}

    \note Die Kommunikationsbibliothek loggt alle Kommunikationsabläufe automatisch in entsprechende Dateien. Daher ist
    nicht notwendig über diese Funktion ein Extralog zu erstellen. Bitte sehen Sie für nähere Informationen
    in der Dokumentation zum Thema Hintergrundloggen nach.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCSetLogOff( int connectionNumber );

/*!
    \brief Setzen des Dateinamens für die Logfunktionalität der Verbindungsnummer.

    Bekanntgeben der Datei die fuer das Loggen verwendet wird.

    \ingroup log

    \export{_DFCSetLogFileName@8, DFCSetLogFileName}

    \note Die Kommunikationsbibliothek loggt alle Kommunikationsabläufe automatisch in entsprechende Dateien. Daher ist
    nicht notwendig über diese Funktion ein Extralog zu erstellen. Bitte sehen Sie für nähere Informationen
    in der Dokumentation zum Thema Hintergrundloggen nach.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] filePath Zeichenfolge des Pfades und/oder Dateinamens.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCSetLogFileName( int connectionNumber, const char *filePath );

#endif
