/**
* @file  file_touch_keys.h
* @brief Schnittstelle fuer die Klasse FileTouchKeys.
*/
//////////////////////////////////////////////////////////////////////
#pragma once
#ifndef FILE_TOUCH_KEYS_H
#define FILE_TOUCH_KEYS_H

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

// Verwaltungsklasse fuer Datei
#include "file_intel_hex.h"

// Strukturdefinitionen
#include "interface/touch_keys_structs.h"

#include <array>

class FileTouchKeys : public FileIntelHex
{
public:
    FileTouchKeys();
    virtual ~FileTouchKeys();

public:
    enum { max_touch_keys = 100 };

    // Verwaltungsstrukturen fuer TouchKey
    TOUCHKEYNODE_V1 *m_touchKeysNode_v1{};
    TOUCHKEYNODE_V2 *m_touchKeysNode_v2{};
    std::array<TOUCHKEY_V1 *, max_touch_keys> m_touchKeys_v1{};
    std::array<TOUCHKEY_V2 *, max_touch_keys> m_touchKeys_v2{};

public:
    void resetData();
    bool load( const std::string &filePath );
    bool save( const std::string &filePath, bool appendToFile = false, bool appendEndLine = true );

    void init( uint8_t count, uint8_t deviceType );
    
    void setTouchKey( uint8_t idx, const TOUCHKEY_V1 &tk );
    void setTouchKey( uint8_t idx, const TOUCHKEY_V2 &tk );
    const TOUCHKEY_V1 &touchKey_v1( uint8_t idx ) const;
    const TOUCHKEY_V2 &touchKey_v2( uint8_t idx ) const;
    
    uint8_t touchKeyCount() const;
    uint8_t deviceType() const;
    void setDeviceType( uint8_t deviceType );
    uint8_t version() const;
};

#endif

