#include "file/interface/dfitem/dfitem_device.h"

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CDFItemDevice::CDFItemDevice( tinyxml2::XMLDocument *doc ) : CDFItemBase( doc )
{
    xmlElement()->SetText( "ItemDevice" );
}

CDFItemDevice::~CDFItemDevice()
{

}

int CDFItemDevice::Modify( CModifyData &modifyData )
{
    StructConfig &config = modifyData.file->m_config;

    // Geraetetyp ersetzen?
    if ( modifyData.flags & SMDF_DEVICE_TYPE )
        modifyData.file->m_flash_header.setupType = modifyData.setupType;

    // Systempasswort ersetzen?
    if ( modifyData.flags & SMDF_PASSWORD_SYSTEM )
    {
        config.passwordSystemMenu = (unsigned short)modifyData.passwordSystem;
        ChangeOrderInt( (unsigned char *)&config.passwordSystemMenu );
    }
    // Benutzerpasswort ersetzen?
    if ( modifyData.flags & SMDF_PASSWORD_USER )
    {
        config.transponderUserPassword = (unsigned short)modifyData.passwordUser;
        ChangeOrderInt( (unsigned char *)&config.transponderUserPassword );
    }

    // Erweiterten Struktur lesen
    StructExtendedFlash &extBegin = *(StructExtendedFlash *)modifyData.file->m_flashExtended;
    StructExtended extended;

    // Lesen der erweiterten Parameter - Struktur
    // Diese Struktur enthaelt informationen auf die beim Baumaufbau von anderen Eintraegen zurueckgegriffen wird,
    // somit muss sie vor dem Aufbau geladen sein.
    if ( extBegin.offsetExtendedStruct )
    {
        extended = *(StructExtended *)&modifyData.file->m_flashExtended[extBegin.offsetExtendedStruct];
    }

    // Transpondertype und Zusatzparameter setzen
    if ( modifyData.flags & SMDF_TRANSPONDER_TECHNIC )
    {
        switch ( modifyData.technic )
        {
            case TRANSPONDER_UNIQUE: // Unique 
            {
                // Transpondertyp: Unique
                config.transponderType = 0;

                // 32bit oder 40bit
                if ( modifyData.flagsTranponder & TMDF_UNIQUE )
                    config.transponderModeUnique = modifyData.unique;

                // Datenformat
                if ( modifyData.flagsTranponder & TMDF_UNIQUE_MODE )
                    config.presentationModeUnique = modifyData.uniqueMode;

                break;
            }
            case TRANSPONDER_HITAG2: // Hitag2
            {
                // Transpondertyp: Hitag2
                config.transponderType = 0x81;

                // Passwort
                if ( modifyData.flagsTranponder & TMDF_PASSWORD )
                    strcpy( extended.transponderPassword32, modifyData.hitagPassword );

                // Wert fuer Increment
                if ( modifyData.flagsTranponder & TMDF_AUTOINC_VALUE )
                    config.autoIncrementValue = modifyData.hitagAutoincValue;

                // Block 1
                // Segment
                if ( modifyData.flagsTranponder & TMDF_SEGMENT1 )
                    config.transponderGlobal.tpSegment1 = modifyData.transponder.tpSegment1;

                // Format oder Laenge
                if ( modifyData.flagsTranponder & TMDF_MODE1_FORMAT || modifyData.flagsTranponder & TMDF_MODE1_LENGTH )
                    config.transponderGlobal.tpMode1 = modifyData.transponder.tpMode1;

                // Initialisierungswert
                if ( modifyData.flagsTranponder & TMDF_INIT_VALUE1 )
                    config.initialValue1 = modifyData.hitagInitValue1;

                // Flag fuer Autoinc
                if ( modifyData.flagsTranponder & TMDF_AUTOINC_FLAG1 )
                {
                    if ( modifyData.hitagAutoincFlags & 0x01 )
                        config.autoIncrementFlags |= 0x01;
                    else
                        config.autoIncrementFlags &= ~0x01;
                }
                // Block 2
                // Segment
                if ( modifyData.flagsTranponder & TMDF_SEGMENT2 )
                    config.transponderGlobal.tpSegment2 = modifyData.transponder.tpSegment2;

                // Format oder Laenge
                if ( modifyData.flagsTranponder & TMDF_MODE2_FORMAT || modifyData.flagsTranponder & TMDF_MODE2_LENGTH )
                    config.transponderGlobal.tpMode2 = modifyData.transponder.tpMode2;

                // Initialisierungswert
                if ( modifyData.flagsTranponder & TMDF_INIT_VALUE2 )
                    config.initialValue2 = modifyData.hitagInitValue2;

                // Flag fuer Autoinc
                if ( modifyData.flagsTranponder & TMDF_AUTOINC_FLAG2 )
                {
                    if ( modifyData.hitagAutoincFlags & 0x02 )
                        config.autoIncrementFlags |= 0x02;
                    else
                        config.autoIncrementFlags &= ~0x02;
                }
                // Block 3
                // Segment
                if ( modifyData.flagsTranponder & TMDF_SEGMENT3 )
                    config.transponderGlobal.tpSegment3 = modifyData.transponder.tpSegment3;

                // Format oder Laenge
                if ( modifyData.flagsTranponder & TMDF_MODE3_FORMAT || modifyData.flagsTranponder & TMDF_MODE3_LENGTH )
                    config.transponderGlobal.tpMode3 = modifyData.transponder.tpMode3;

                // Initialisierungswert
                if ( modifyData.flagsTranponder & TMDF_INIT_VALUE3 )
                    config.initialValue3 = modifyData.hitagInitValue3;

                // Flag fuer Autoinc
                if ( modifyData.flagsTranponder & TMDF_AUTOINC_FLAG3 )
                {
                    if ( modifyData.hitagAutoincFlags & 0x04 )
                        config.autoIncrementFlags |= 0x04;
                    else
                        config.autoIncrementFlags &= ~0x04;
                }

                break;
            }
            case TRANSPONDER_LEGIC:
            {
                // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;

                // Unikatsnummer lesen
                config.transponderType |= 0x40;
                config.transponderSegment = 0;
                config.transponderCutFlags = 0x01;
                config.transponderCutFrom = 0;
                config.transponderCutLength = 5;
                break;
            }
            case TRANSPONDER_MIFARE: // Mifare
            {
                // Transpondertyp: Mifare
                if ( config.transponderType != 0xC5 && config.transponderType != 0x85 )
                    config.transponderType = 0x85;

                // Referenz auf Struktur des Configbereichs
                StructTransponderMifare &tpGlobal = *(StructTransponderMifare *)&config.transponderGlobal;

                // Wert fuer Seriennummer lesen
                if ( modifyData.flagsTranponder & TMDF_SERIAL_NUMBER )
                {
                    if ( modifyData.serial_number == 1 )
                        config.transponderType |= 0x40;
                    else
                        config.transponderType &= ~0x40;
                }

                // Wert fuer Increment
                if ( modifyData.flagsTranponder & TMDF_AUTOINC_VALUE )
                    config.autoIncrementValue = modifyData.hitagAutoincValue;

                // Segment angegeben
                if ( modifyData.flagsTranponder & TMDF_SECTOR )
                {
                    tpGlobal.sectorBlock &= ~0xF0;
                    tpGlobal.sectorBlock |= (modifyData.transponderMifare.sectorBlock & 0xF0);
                }


                // Block angegeben
                if ( modifyData.flagsTranponder & TMDF_BLOCK )
                {
                    tpGlobal.sectorBlock &= ~0x0F;
                    tpGlobal.sectorBlock |= (modifyData.transponderMifare.sectorBlock & 0x0F);
                }

                // Modusvariable ggf. initialisieren
                if ( modifyData.flagsTranponder & TMDF_MODE_FORMAT || modifyData.flagsTranponder & TMDF_MODE_LENGTH )
                    tpGlobal.mode |= BDTP_ACTIVE;

                // Format in Modusvariable uebernehmen
                if ( modifyData.flagsTranponder & TMDF_MODE_FORMAT )
                {
                    tpGlobal.mode &= ~0x30;
                    tpGlobal.mode |= (modifyData.transponderMifare.mode & 0x30);
                }

                // Format in Modusvariable uebernehmen
                if ( modifyData.flagsTranponder & TMDF_MODE_LENGTH )
                {
                    tpGlobal.mode &= ~0x0F;
                    tpGlobal.mode |= (modifyData.transponderMifare.mode & 0x0F);
                }

                // Startposition uebernehmen
                if ( modifyData.flagsTranponder & TMDF_POSITION )
                {
                    tpGlobal.cut &= ~0xF0;
                    tpGlobal.cut = (modifyData.transponderMifare.cut & 0xF0);
                }

                int i, len = (int)strlen( modifyData.mifareInitalValue );
                // Initialisierungswert angegeben
                if ( modifyData.flagsTranponder & TMDF_INIT_VALUE1 )
                {
                    switch ( tpGlobal.mode & 0x30 )
                    {
                        case 0: // Dez
                            *(unsigned int *)&extended.transponderInitialValue = (unsigned int)atoi( modifyData.mifareInitalValue );
                            break;

                        case 1: // Hex
                        case 3: // Hex gedreht
                            for ( i = 0; i < len; ++i )
                            {
                                if ( !isxdigit( (int)modifyData.mifareInitalValue[i] ) )
                                    break;
                            }

                            if ( i == len )
                                *(unsigned int *)&extended.transponderInitialValue = HexToUINT32( modifyData.mifareInitalValue );
                            else
                                *(unsigned int *)&extended.transponderInitialValue = 0;

                            break;

                        case 2: // ASCII
                            memset( extended.transponderInitialValue, 0, sizeof( extended.transponderInitialValue ) );
                            strcpy( (char *)extended.transponderInitialValue, modifyData.mifareInitalValue );
                            break;
                    }
                }

                // Flag fuer Autoinc
                if ( modifyData.flagsTranponder & TMDF_AUTOINC_FLAG1 )
                {
                    if ( modifyData.hitagAutoincFlags & 0x01 )
                        config.autoIncrementFlags |= 0x01;
                    else
                        config.autoIncrementFlags &= ~0x01;
                }


                // Passwortgruppe 1
                // Passwortmodus angegeben
                if ( modifyData.flagsPassword & PMDF_MODUS1 )
                {
                    extended.transponderPasswordFlag &= ~0x03;
                    extended.transponderPasswordFlag |= (modifyData.passwordFlags & 0x03);
                }
                // Passwort angegeben
                if ( modifyData.flagsPassword & PMDF_KEYA0 )
                    strcpy( extended.transponderPasswordA0, modifyData.mifarePasswordA0 );

                // Passwort angegeben
                if ( modifyData.flagsPassword & PMDF_KEYB0 )
                    strcpy( extended.transponderPasswordB0, modifyData.mifarePasswordB0 );

                // Passwortgruppe 2
                // Passwortmodus angegeben
                if ( modifyData.flagsPassword & PMDF_MODUS2 )
                {
                    extended.transponderPasswordFlag &= ~(0x03 << 2);
                    extended.transponderPasswordFlag |= (modifyData.passwordFlags & (0x03 << 2));
                }
                // Passwort angegeben
                if ( modifyData.flagsPassword & PMDF_KEYA1 )
                    strcpy( extended.transponderPasswordA1, modifyData.mifarePasswordA1 );

                // Passwort angegeben
                if ( modifyData.flagsPassword & PMDF_KEYB1 )
                    strcpy( extended.transponderPasswordB1, modifyData.mifarePasswordB1 );

                // Passwortgruppe 3
                // Passwortmodus angegeben
                if ( modifyData.flagsPassword & PMDF_MODUS3 )
                {
                    extended.transponderPasswordFlag &= ~(0x03 << 4);
                    extended.transponderPasswordFlag |= (modifyData.passwordFlags & (0x03 << 4));
                }
                // Passwort angegeben
                if ( modifyData.flagsPassword & PMDF_KEYA2 )
                    strcpy( extended.transponderPasswordA2, modifyData.mifarePasswordA2 );

                // Passwort angegeben
                if ( modifyData.flagsPassword & PMDF_KEYB2 )
                    strcpy( extended.transponderPasswordB2, modifyData.mifarePasswordB2 );

                break;
            }
            case TRANSPONDER_IBUTTON:
            {
                // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;

                break;
            }
            case TRANSPONDER_SIMONSVOSS:
            {
                // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;

                break;
            }
            case TRANSPONDER_NEDAP:
            {
                // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;

                break;
            }
            case TRANSPONDER_HID_PROXKEY: // HID-Proxkey
            case TRANSPONDER_HID_ICLASS: // HID-iClass
            {
              // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;

                // Anpassung des Typ uebernehmen
                if ( modifyData.flagsTranponder & TMDF_HID_TYPE )
                    ((StructTransponderHIDiClass *)&config.transponderGlobal)->type = modifyData.transponderHID.type;

                // Anpassung des Format uebernehmen
                if ( modifyData.flagsTranponder & TMDF_HID_FORMAT )
                    ((StructTransponderHIDiClass *)&config.transponderGlobal)->format = modifyData.transponderHID.format;

                // Anpassung der Laenge uebernehmen
                if ( modifyData.flagsTranponder & TMDF_HID_LENGTH )
                    ((StructTransponderHIDiClass *)&config.transponderGlobal)->length = modifyData.transponderHID.length;

                break;
            }

            case TRANSPONDER_WIEGAND: // Wiegand
            case TRANSPONDER_CLOCK_DATA: // Clock Data 
            {
                // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;
                // ToDo Anpassungen in der Struktur übernehmen
                // Benoetigte Mindestversion
                //modifyData.file->NeedStudioVersion("04.02.05.01");
                //modifyData.file->NeedFirmware2Version("04.02.05.01");
                //modifyData.file->UnsupportFirmware1Version();
                // Gesamte Transponderkonfigurationen in den Erweitertbereich ablegen.

                // Eintragen der gesamten Konfigurationen in den Extended-Bereich.
                extBegin.offsetTransponderConfiguration = extBegin.offsetFreeScope;

                // Feste Werte sicherstellen und eintragen.
                modifyData.m_transponderWiegandClockDataHeader.structSize = sizeof( modifyData.m_transponderWiegandClockDataHeader );
                modifyData.m_transponderWiegandClockDataHeader.structFlags = 0x01; // Ggf. verwendete Passwoerter mit RC4 und DatafoxKey verschluesseln.
                modifyData.m_transponderWiegandClockDataHeader.transponderType = (modifyData.technic & 0x3F);
                // Headerstruktur eintragen
                *(StructTransponderWiegandClockDataHeader *)&modifyData.file->m_flashExtended[extBegin.offsetTransponderConfiguration] = modifyData.m_transponderWiegandClockDataHeader;
                // Basiszeiger und Zeiger auf OffsetArray
                unsigned char *base = &modifyData.file->m_flashExtended[extBegin.offsetTransponderConfiguration + sizeof( StructTransponderWiegandClockDataHeader )];
                unsigned short *offsArray = (unsigned short *)base;
                // Zeiger auf die erste Transponderkonfiguration.
                StructTransponderWiegandClockData *transponderStruct = (StructTransponderWiegandClockData *)(base + modifyData.m_transponderWiegandClockDataHeader.count * sizeof( unsigned short ));

                for ( int i = 0; i < modifyData.m_transponderWiegandClockDataHeader.count; i++ )
                {

                    // Offset und Transponderstruktur eintragen.
                    *offsArray = (unsigned short)((unsigned char *)transponderStruct - base);
                    *transponderStruct = modifyData.m_transponderWiegandClockData[i];

                    // Beide Zeiger um eins weiter.
                    offsArray++;
                    transponderStruct++;
                }

                // Positionsoffset des naechsten freien Speichers berechnen.
                extBegin.offsetFreeScope += sizeof( StructTransponderWiegandClockDataHeader ) + modifyData.m_transponderWiegandClockDataHeader.count * sizeof( unsigned short ) + modifyData.m_transponderWiegandClockDataHeader.count * sizeof( StructTransponderWiegandClockData );
                break;
            }
            case TRANSPONDER_BLUEBOX_ID_TRONIC:
            {
                // Transpondertyp uebernehmen
                config.transponderType = modifyData.technic;

                // Benoetigte Mindestversion
                modifyData.file->NeedStudioVersion( "04.02.05.01" );
                modifyData.file->NeedFirmware2Version( "04.02.05.01" );
                modifyData.file->UnsupportFirmware1Version();

                // Gesamte Transponderkonfigurationen in den Erweitertbereich ablegen.

                // Eintragen der gesamten Konfigurationen in den Extended-Bereich.
                extBegin.offsetTransponderConfiguration = extBegin.offsetFreeScope;

                // Feste Werte sicherstellen und eintragen.
                modifyData.m_transponderBlueboxIDTronicHeader.structSize = sizeof( modifyData.m_transponderBlueboxIDTronicHeader );
                modifyData.m_transponderBlueboxIDTronicHeader.structFlags = 0x01; // Ggf. verwendete Passwoerter mit RC4 und DatafoxKey verschluesseln.
                modifyData.m_transponderBlueboxIDTronicHeader.transponderType = (modifyData.technic & 0x3F);
                // Headerstruktur eintragen
                *(StructTransponderETSITagsHeader *)&modifyData.file->m_flashExtended[extBegin.offsetTransponderConfiguration] = modifyData.m_transponderBlueboxIDTronicHeader;
                // Basiszeiger und Zeiger auf OffsetArray
                unsigned char *base = &modifyData.file->m_flashExtended[extBegin.offsetTransponderConfiguration + sizeof( StructTransponderETSITagsHeader )];
                unsigned short *offsArray = (unsigned short *)base;
                // Zeiger auf die erste Transponderkonfiguration.
                StructTransponderETSITags *transponderStruct = (StructTransponderETSITags *)(base + modifyData.m_transponderBlueboxIDTronicHeader.count * sizeof( unsigned short ));

                int i;
                for ( i = 0; i < modifyData.m_transponderBlueboxIDTronicHeader.count; i++ )
                {
                    // Offset und Transponderstruktur eintragen.
                    *offsArray = (unsigned short)((unsigned char *)transponderStruct - base);
                    *transponderStruct = modifyData.m_transponderBlueboxIDTronic[i];

                    // Beide Zeiger um eins weiter.
                    offsArray++;
                    transponderStruct++;
                }

                // Positionsoffset des naechsten freien Speichers berechnen.
                extBegin.offsetFreeScope += sizeof( StructTransponderETSITagsHeader ) + modifyData.m_transponderBlueboxIDTronicHeader.count * sizeof( unsigned short ) + modifyData.m_transponderBlueboxIDTronicHeader.count * sizeof( StructTransponderETSITags );
                break;
            }
            case TRANSPONDER_NONE: // Keinen    
            {
                // Transpondertyp: Keinen
                config.transponderType = TRANSPONDER_NONE;
                break;
            }
        }
    }

    // Sicherheitslevel ersetzen?
    if ( modifyData.flags & SMDF_FINGERPRINT_SECURITY_LEVEL )
    {
        extended.fingerprint.securityLevel = modifyData.securityLevel;
        extended.fingerprint.securityFRR = modifyData.securityFRR;
        extended.fingerprint.securityFAR = modifyData.securityFAR;
    }
    // Mindestqualitaet beim scannnen ersetzen?
    if ( modifyData.flags & SMDF_FINGERPRINT_QUALITY )
        extended.fingerprint.quality = modifyData.quality;

    // Mindestanzahl der Minutiae beim scannen ersetzen?
    if ( modifyData.flags & SMDF_FINGERPRINT_MINUTIAE_COUNT )
        extended.fingerprint.minutiaeCount = modifyData.minutiaeCount;

    // Schreiben der erweiterten Parameter - Struktur
    if ( extBegin.offsetExtendedStruct )
        *(StructExtended *)&modifyData.file->m_flashExtended[extBegin.offsetExtendedStruct] = extended;

   // Schreiben der Erweiterten Struktur
    *(StructExtendedFlash *)modifyData.file->m_flashExtended = extBegin;

    unsigned int offs;
    // Eintraege hinzufuegen
    //#####################################################
    // Eintrag fuer Datensatzbeschreibung einbinden
    CDFItemBase *item = new CDFItemTableRecord( xmlElement()->GetDocument() );
    if ( item == NULL )
        return 0;

    xmlElement()->InsertEndChild( item->xmlElement() );
    // Datensatzbeschreibungen lesen
    offs = modifyData.file->endian_reverse( modifyData.file->m_flash_header.dataStruct );
    modifyData.offsFlash = offs;
    if ( !item->Modify( modifyData ) )
        return 0;

    // Eintrag fuer Listenbeschreibung einbinden
    item = new CDFItemTableList( xmlElement()->GetDocument() );
    if ( item == NULL )
        return 0;

    xmlElement()->InsertEndChild( item->xmlElement() );
    // Listenbeschreibungen lesen
    offs = modifyData.file->endian_reverse( modifyData.file->m_flash_header.listStruct );
    modifyData.offsFlash = offs;
    if ( !item->Modify( modifyData ) )
        return 0;

    // Eintrag fuer Bedienung einbinden
    item = new CDFItemHandling( xmlElement()->GetDocument() );
    if ( item == NULL )
        return 0;

    xmlElement()->InsertEndChild( item->xmlElement() );
    StructGlobalVar globVar = *(StructGlobalVar *)modifyData.file->m_flash_mem.data();
    ChangeOrderLong( (unsigned char *)&globVar.ptr );
    modifyData.offsFlash = globVar.ptr;
    if ( !item->Modify( modifyData ) )
        return 0;

    // Eintrag fuer Signalverarbeitung einbinden
    item = new CDFItemMDE( xmlElement()->GetDocument() );
    if ( item == NULL )
        return 0;

    xmlElement()->InsertEndChild( item->xmlElement() );
    modifyData.offsFlash = 0;
    if ( !item->Modify( modifyData ) )
        return 0;

    // Eintrag fuer Zutrittskontrolle einbinden
    item = new CDFItemAccessControl( xmlElement()->GetDocument() );
    if ( item == NULL )
        return 0;

    xmlElement()->InsertEndChild( item->xmlElement() );
    modifyData.offsFlash = 0;
    if ( !item->Modify( modifyData ) )
        return 0;

    // Eintrag fuer Zutrittskontrolle 2 einbinden
    item = new CDFItemAccessControl2( xmlElement()->GetDocument() );
    if ( item == NULL )
        return 0;

    xmlElement()->InsertEndChild( item->xmlElement() );
    modifyData.offsFlash = 0;
    if ( !item->Modify( modifyData ) )
        return 0;

    return 1;
}
